io_uring_prep_read

Function io_uring_prep_read 

pub unsafe fn io_uring_prep_read(
    sqe: *mut io_uring_sqe,
    fd: c_int,
    buf: *mut c_void,
    nbytes: c_uint,
    offset: u64,
)
Prepare I/O read request

DESCRIPTION

The io_uring_prep_read prepares an IO read request. The submission queue entry sqe is setup to use the file descriptor fd to start reading nbytes into the buffer buf at the specified offset.

On files that support seeking, if the offset is set to -1, the read operation commences at the file offset, and the file offset is incremented by the number of bytes read. See read for more details. Note that for an async API, reading and updating the current file offset may result in unpredictable behavior, unless access to the file is serialized. It is not encouraged to use this feature, if it’s possible to provide the desired IO offset from the application or library.

On files that are not capable of seeking, the offset must be 0 or -1.

After the read has been prepared it can be submitted with one of the submit functions.

RETURN VALUE

None

ERRORS

The CQE res field will contain the result of the operation. See the related man page for details on possible values. Note that where synchronous system calls will return -1 on failure and set errno to the actual error value, io_uring never uses errno. Instead it returns the negated errno directly in the CQE res field.

NOTES

This function accepts an unsigned number of bytes, but io_uring_cqe’s result code is an __s32 value, so in theory a short read with a large enough nbytes value could generate an ambiguous return. But the number of bytes actually transferred has the same limit as read so this cannot happen in practice.

SEE ALSO

io_uring_get_sqe, io_uring_prep_readv, io_uring_prep_readv2, io_uring_submit