pub unsafe extern "C" fn io_uring_queue_exit(ring: *mut io_uring)Expand description
Tear down io_uring submission and completion queues
DESCRIPTION
io_uring_queue_exit will release all resources acquired and initialized by io_uring_queue_init. It first unmaps the memory shared between the application and the kernel and then closes the io_uring file descriptor.
Some accounting is done asynchronously, so memory locked by io_uring_queue_init may remain locked for a few milliseconds after this function returns.
RETURN VALUE
None